/*
 * Copyright 2017 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.vms.web;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.regex.Pattern;

import javax.servlet.http.HttpServletRequest;

import dk.vajhoej.record.RecordException;
import dk.vajhoej.vms.call.lib.SYS;

/**
 * Class Util contains various utility methods.
 */
public class Util {
	/**
	 * Read binary file.
	 * @param fnm file name
	 * @return file content
	 * @throws IOException if error reading file
	 */
	public static byte[] loadBinary(String fnm) throws IOException {
		InputStream is = new FileInputStream(fnm);
		byte[] buf = new byte[10000000];
		int ix = 0;
		int n;
		while((n = is.read(buf, ix, buf.length - ix)) > 0) {
			ix += n;
		}
		is.close();
		if(ix >= buf.length) {
			throw new IOException("File too large");
		}
		byte[] res = new byte[ix];
		System.arraycopy(buf, 0, res, 0 , ix);
		return res;
	}
	/**
	 * Read text file.
	 * @param fnm file name
	 * @return file content
	 * @throws IOException if error reading file
	 */
	public static String loadText(String fnm) throws IOException {
		BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(fnm), "UTF-8"));
		StringBuilder sb = new StringBuilder();
		String line;
		while((line = br.readLine()) != null) {
			sb.append(line);
			sb.append("\r\n");
		}
		br.close();
		return sb.toString();
	}
	/**
	 * Check if HTTP request seems valid.
	 * @param remurl remainding URL
	 * @param req HTTP request object
	 * @return whether valid
	 */
	public static boolean isValidRequest(String remurl, HttpServletRequest req) {
		if(!req.getMethod().equals("GET")) return false;
		if(remurl.length() > 255) return false;
		if(req.getQueryString() != null && req.getQueryString().length() > 255) return false;
		return true;
	}
	private static final Pattern OKFNM = Pattern.compile("/[A-Za-z0-9_][A-Za-z0-9_./]*");
	/**
	 * Check if URL maps to a valid file under base directory.
	 * @param dir base directory
	 * @param remurl remaining URL to be appended to base directory
	 * @return whether valid
	 * @throws IOException if error accessing directories/file
	 */
	public static boolean isValidFile(String dir, String remurl) throws IOException {
		File d = new File(dir);
		if(!d.exists()) return false;
		if(!d.isDirectory()) return false;
		if(!OKFNM.matcher(remurl).matches()) return false;
		String fnm = dir + remurl;
		File f = new File(fnm);
		if(!f.exists()) return false;
		if(!f.getCanonicalPath().startsWith(dir)) return false;
		return true;
	}
	/**
	 * Check credentials (username + password).
	 * @param username username
	 * @param password password
	 * @return whether valid login
	 * @throws UnsupportedEncodingException if error with character set
	 * @throws RecordException if internal error
	 */
	public static boolean checkCredentials(String username, String password) throws UnsupportedEncodingException, RecordException {
		return SYS.SYS$ACMW(username, password).getResult();
	}
}
