/*
 * Copyright 2017 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.vms.web.http;

import java.io.IOException;
import java.util.Collections;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.eclipse.jetty.security.ConstraintMapping;
import org.eclipse.jetty.security.ConstraintSecurityHandler;
import org.eclipse.jetty.security.authentication.BasicAuthenticator;
import org.eclipse.jetty.server.Connector;
import org.eclipse.jetty.server.Request;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.handler.AbstractHandler;
import org.eclipse.jetty.server.handler.ContextHandler;
import org.eclipse.jetty.server.handler.ContextHandlerCollection;
import org.eclipse.jetty.server.nio.SelectChannelConnector;
import org.eclipse.jetty.util.security.Constraint;

import dk.vajhoej.vms.web.AccessModule;
import dk.vajhoej.vms.web.FunctionModule;
import dk.vajhoej.vms.web.Result;
import dk.vajhoej.vms.web.Util;

/**
 * Class HttpModule implements HTTP protocol.
 */
public class HttpModule implements AccessModule {
	static class FunctionHandler extends AbstractHandler {
		private FunctionModule fm;
		public FunctionHandler(FunctionModule fm) {
			this.fm = fm;
		}
		public void handle(String target, Request base, HttpServletRequest req, HttpServletResponse resp) throws IOException, ServletException {
			if(!Util.isValidRequest(target, req)) {
				System.err.println("Invalid HTTP request for: " + target);
				resp.setStatus(HttpServletResponse.SC_BAD_REQUEST);
			}
			Result r = fm.getResponse(target, req.getQueryString());
			if(r.isStatus()) {
				resp.setStatus(HttpServletResponse.SC_OK);
				resp.setContentType(r.getContentType());
				resp.getOutputStream().write(r.getContentBody());
			} else {
				System.err.println(r.getErrorMessage());
				resp.setStatus(HttpServletResponse.SC_BAD_REQUEST);
			}
			base.setHandled(true);
		}
	}
	private int port;
	/**
	 * Create instance.
	 * @param port port
	 */
	public HttpModule(int port) {
		this.port = port;
	}
	protected Connector getConnector(int port) {
		Connector con = new SelectChannelConnector();
		con.setPort(port);
		return con;
	}
	protected boolean isLoginValid() {
		return System.getProperty("I.know.what.I.am.doing") != null ? true : false;
	}
	public void run(List<FunctionModule> funcmod, boolean login) {
		if(login && !isLoginValid()) {
			System.err.println("Login not enabled for insecure transport");
			return;
		}
		try {
			Server srv = new Server();
			srv.setConnectors(new Connector[] { getConnector(port) });
			ContextHandlerCollection allctx = new ContextHandlerCollection();
			for(FunctionModule fm : funcmod) {
				ContextHandler ctx = new ContextHandler();
				ctx.setContextPath("/" + fm.getContext());
				ctx.setHandler(new FunctionHandler(fm));
				allctx.addHandler(ctx);
			}
			if(login) {
		        Constraint constraint = new Constraint();
		        constraint.setAuthenticate(true);
		        constraint.setRoles(new String[] { "user" });
		        ConstraintMapping mapping = new ConstraintMapping();
		        mapping.setPathSpec("/*");
		        mapping.setConstraint(constraint);
		        ConstraintSecurityHandler security = new ConstraintSecurityHandler();
		        security.setConstraintMappings(Collections.singletonList(mapping));
		        security.setAuthenticator(new BasicAuthenticator());
		        security.setLoginService(new VmsLoginService());
		        security.setHandler(allctx);
		        srv.setHandler(security);
			} else {
				srv.setHandler(allctx);
			}
			srv.start();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}

